#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2019 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>


from gimpfu import *
import os
import math
import random

from gtk import gdk
screen_width = gdk.screen_width()
screen_height = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)

my_error = "my_error = " #Global message collector


def do_stuff(image, layer, folder_info, allow_before, allow_after, text, horizontal_padding, vertical_padding, num_repeats, placement_style, add_drop, drop_offset, drop_blur, add_shuffle):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)	
    use_custom_font = True
    use_random_font = True
    colour_style = 0
    force_it = False # for checking
    if num_repeats<1:num_repeats = 1
	
    if add_shuffle:
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        layer_names_list = image.layers
        loop = 0
        shuffle = num_layers - 1
        if shuffle > 1:
            while loop < num_layers -1:
                random.seed()
                iloop = 0
                deal = random.randrange(0,num_layers -2)
                random.seed()
                pdb.gimp_image_lower_item(image, layer_names_list[deal])
                num_layers, layer_ids = pdb.gimp_image_get_layers(image)
                layer_names_list = image.layers
                loop += 1
    else:
        if text == "": text = "Gimp"
        use_fg_colour, use_bg_colour, use_gradient_colouring, use_random_gradient_colour, cross = False, False, False, False, False
        if colour_style == 0:
           use_fg_colour = True
        if colour_style == 1:
           use_bg_colour = True
        if colour_style == 2:
           use_gradient_colouring = True
        if colour_style == 3:
           use_random_gradient_colour = True
		   
        gradient_colour = pdb.gimp_context_get_gradient()
        current_gradient = gradient_colour
        list_of_gradient_positions = fn_get_gradient_positions(num_repeats)
        number_of_samples, colours_of_samples = pdb.gimp_gradient_get_custom_samples(current_gradient, len(list_of_gradient_positions), list_of_gradient_positions,False)
	
        chars_in_ui_text = fn_chars_in_text(text)
        chars_in_ui_text = chars_in_ui_text.decode('utf-8')
	
        fontimg = -1
        xcffilename = ""
	
        if use_custom_font:
            if str(folder_info) == "None": 
                use_custom_font = False
                num_repeats = 1
                placement_style = 0
                text = "Gimp loves you"
                old_handler = pdb.gimp_message_get_handler()	
                pdb.gimp_message_set_handler(MESSAGE_BOX)
                pdb.gimp_message("Please select a custom font folder to work with and try again.")
                pdb.gimp_message_set_handler(old_handler)
			
	
        if use_custom_font:
            loop, name_check = 0, False
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
            layer_names_list = image.layers
            while loop < num_layers: # check that at least one existing layer has been created with custom fonts
                layer_name = pdb.gimp_item_get_name(layer_names_list[loop])
                layer_name = layer_name.decode('utf-8')
                if "Custom Font" in layer_name: name_check = True
                loop += 1
			
                if not name_check: # if there is no existing custom font layer remove any existing parasite information
                    num_parasites, parasite_list = pdb.gimp_get_parasite_list()
                    if "folderinfo" in parasite_list: pdb.gimp_detach_parasite("folderinfo")
            
        #############################################
		

            num_parasites, parasite_list = pdb.gimp_get_parasite_list()
            if "folderinfo" not in parasite_list: # if our parasite has not been registered/does not yet exist set old folder value to empty
                old_folder_info = ""
            else: # otherwise set old folder value to stored parasite contents
                old_folder_info = str(pdb.gimp_get_parasite("folderinfo"))

                # make a list of the custom font file locations taken from the parasite
                temp_file_list = ["dummy"]
                pos = 6
                new_pos = old_folder_info.find(",", pos)
                while new_pos <> -1:
                    temp_file_list.append(old_folder_info[pos: new_pos])
                    pos = new_pos + 1
                    new_pos = old_folder_info.find(",", pos)
            
                if new_pos == -1: 
                    temp_file_list.append(old_folder_info[pos: len(old_folder_info)])
			
                xcf_file_list = temp_file_list
                num_custom_fonts = len(xcf_file_list) - 1
                font_name_pointer, file_dummy = -1, 0
                f_info = folder_info.rfind("\\") + 1
                temp = folder_info[f_info : len(folder_info)]
				
        ############################################
		
            f_info = folder_info.rfind("\\") + 1
            if folder_info[f_info : len(folder_info)] not in str(old_folder_info): # if there is no custom font folder information we need to gather it from the directory passed to us from the filter dialog
                xcf_file_list = ["dummy"]
                xcf_file_list_string = "dummy"
                for file in os.listdir(folder_info):
                    file_extension = os.path.splitext(file)[1]
                    file_name = os.path.join(folder_info, file)
                    file_path = os.path.splitext(file_name)[0]
                    base_path = os.path.basename(file_name)
			
                    if file_extension.lower() == ".xcf":
                        font_name_pointer, file_dummy = -1, 0
                        fontimg = pdb.gimp_xcf_load(file_dummy,file_name,file_name)
                        num_layers, layer_ids = pdb.gimp_image_get_layers(fontimg)
                        if fn_test_for_custom_chars(fontimg, text):
                            xcf_file_list.append(file_name)
                            xcf_file_list_string += "," + file_name
                            xcffilename = file_name
			
                gimp.attach_new_parasite("folderinfo", 0, str(xcf_file_list_string) )
                old_folder_info = xcf_file_list_string
                num_custom_fonts = len(xcf_file_list) - 1
            if num_custom_fonts<1: use_custom_font = False

        # variables required by proc_text_along_path
        font_size = 100
        kern_amount = 0
        step_size = 0
        do_best_size = True
	
        font_name_pointer = -1
        fg_colour = pdb.gimp_context_get_foreground()
        blend_colour = pdb.gimp_context_get_background()
        use_gradient_colour = False
        spread_colour = False
        keep_layers = False
        keep_paths = False
        use_space_to_change_step_size = False
        compensate_multiplier = 0
        use_text_size_modifier = False
        initial_text_size = 0
        medial_text_size = 0
        final_text_size = 0
        
        position_of_medial = 0
        do_repeat = False
        remaining_path = 0
        reversed_writing = False
        Left_to_Right_path = False
	
        jitter = False
        jitter_range = 0
        tranparency = 100
        reversed_vector = -1
        custom_font_mode = 47 # number 22 in original mode selection menu
	
        rotate_text = False
        user_angle = 0
	
        num_fonts, font_list = pdb.gimp_fonts_get_list("")
        if not use_random_font: current_font_name = font_name
	
        if use_custom_font: # use a randomly selected name if using custom fonts
            use_random_font = True
            if num_custom_fonts>1:
                random.seed()
                loop = random.randrange(1,(num_custom_fonts + 1))
                xcffilename = xcf_file_list[loop]
            else:
                xcffilename = xcf_file_list[1]
            fontimg = pdb.gimp_xcf_load(file_dummy,xcffilename,xcffilename)
        else:
            vertical_adjustment_list = ""
		
        if placement_style == 4: cross = True
        
	
        my_error = "Initialised: "
        loop = 0
        while loop<num_repeats:

        

            if use_random_font: # decide which type of font to use - standard fonts if custom fonts option not selected
                random.seed()
                if use_custom_font: 
                    if num_custom_fonts<num_repeats:
                        if num_custom_fonts>1:
                            custom_ptr = (random.randrange(1,num_repeats) % num_custom_fonts) + 1
                        else:
                            custom_ptr = 1
                    else:
                        if num_custom_fonts>1:
                            custom_ptr = (random.randrange(1,num_custom_fonts + 1))
                        else:
                            custom_ptr = 1
                    if force_it: custom_ptr =  1 # temporary checking
                    xcffilename = xcf_file_list[custom_ptr]
                    my_error += "Main loop = " + str(loop) + " : Current custom font = " + str(xcffilename) + "\n" #### status checking
                    fontimg = pdb.gimp_xcf_load(file_dummy,xcffilename,xcffilename)
                    # build list to hold vertical character widths here
                    vertical_adjustment_list = ["dummy"]
                    if placement_style>0: ################# added for loads of text
                        temp_loop = 0
                        for chara in chars_in_ui_text:
                            temp_loop += 1
                            adj_x0, adj_y0, adj_x1, adj_y1, adj_wd, adj_ht = fn_get_tight_character_ht_info(image, chara, fontimg)
                            if chara == " ":
                                vertical_adjustment_list.append("space.png")
                                vertical_adjustment_list.append(0)
                                vertical_adjustment_list.append(0)
                                vertical_adjustment_list.append(adj_wd)
                                vertical_adjustment_list.append(adj_ht)
                                vertical_adjustment_list.append(adj_wd)
                                vertical_adjustment_list.append(adj_ht)
                            else:
                                vertical_adjustment_list.append(chara + ".png")
                                vertical_adjustment_list.append(adj_x0)
                                vertical_adjustment_list.append(adj_y0)
                                vertical_adjustment_list.append(adj_x1)
                                vertical_adjustment_list.append(adj_y1)
                                vertical_adjustment_list.append(adj_wd)
                                vertical_adjustment_list.append(adj_ht)
						
                        my_error += "vertical_adjustment_list = " + str(vertical_adjustment_list) + "\n"
			
            current_font_name = random.choice(font_list)
		
            # set path co-ordinates according to user direction
		
            if cross == True: # for when both vertical and horizontal text are required
                random.seed()
                placement_style = (random.randrange(1,3)-1) # random choice of horizontal or vertical
            
		
            if placement_style == 0: # Horizontal
		
                random.seed()
                vertical, diagonal = False, False

                #for horizontal text
                max_path_len = image.width
		
                path_length = random.randrange((image.width/2), max_path_len)
		
                random.seed()
                if allow_before:
                    min_pos = (0 - (random.randrange((path_length/10), path_length/5)))
                    max_pos = min_pos + path_length
                    if max_pos>image.width:
                        max_pos = image.width
                        min_pos = max_pos - path_length
                elif allow_after:
                    min_pos = (random.randrange(0,(path_length - (path_length/10))))
                    max_pos = min_pos + path_length
                    if min_pos>image.width:
                        min_pos = random.randrange(0,int(image.width/2))
                        max_pos = min_pos + path_length
                else:
                    min_pos = (random.randrange(0,(path_length - (path_length/10))))
                    max_pos = min_pos + path_length
                    if max_pos>image.width:
                        max_pos = image.width
                        min_pos = max_pos - path_length
					
		
                path_min = min(min_pos, max_pos)
                path_max = max(min_pos, max_pos)
                if path_min == path_max: path_max += 100
                if path_max <= 0: path_max = 100
	
                # for horizontal letters path
    
                random.seed()
                path_y0 = random.randrange(0, image.height)
                path_y1 = path_y0
                path_x0 = path_min
                path_x1 = path_max
                path_length = path_x1 - path_x0

            
                rotate_text = False
			
            if placement_style == 1: # Vertical
                diagonal = False
                random.seed()
                vertical = True
                #for Vertical text
                max_path_len = image.height
                if allow_before: max_path_len *= 2
                if allow_after: max_path_len *= 3
		
                path_length = random.randrange((image.height/2), max_path_len)
			
                random.seed()
                if allow_before:
                    min_pos = (0 - (random.randrange((path_length/10), path_length/5)))
                    max_pos = min_pos + path_length
                    if max_pos>image.height:
                        max_pos = image.height
                        min_pos = max_pos - path_length
                elif allow_after:
                    min_pos = (random.randrange(0,(path_length - (path_length/10))))
                    max_pos = min_pos + path_length
                    if min_pos>image.height:
                        min_pos = random.randrange(0,int(image.height/2))
                        max_pos = min_pos + path_length
                else:
                    min_pos = (random.randrange(0,(path_length - (path_length/10))))
                    max_pos = min_pos + path_length
                    if max_pos>image.height:
                        max_pos = image.height
                        min_pos = max_pos - path_length
		
		
                path_min = min(min_pos, max_pos)
                path_max = max(min_pos, max_pos)
                if path_min == path_max: path_max += 100
                if path_max <= 0: path_max = 100
			
                # for vertical letters path
    
                random.seed()
                path_y0 = path_min
                path_y1 = path_max
                path_x0 = random.randrange(0, image.height)
                path_x1 = path_x0
                path_length = path_y1 - path_y0

                rotate_text = True
                user_angle = 0
			
            if placement_style == 2: # Diagonal - different approach: Set x1 and y1 coordinates to create path at 45 degrees increasing the distance according to user choices
                diagonal = True
                path_x0 = 0
                path_y0 = 0
                # Ensure that initial maximum path length will fall within the image
                if image.height>image.width:
                    path_x1 = image.width
                    path_y1 = image.width
                else:
                    path_x1 = image.height
                    path_y1 = image.height
			
                if allow_before: # increase potential length of path when allowing text placement to partly fall outside image boundaries
                    if image.height>image.width:
                        path_x1 = image.width * 2
                        path_y1 = image.width * 2
                    else:
                        path_x1 = image.height * 2
                        path_y1 = image.height * 2
                if allow_after: 
                    if image.height>image.width:
                        path_x1 = image.width * 3
                        path_y1 = image.width * 3
                    else:
                        path_x1 = image.height * 3
                        path_y1 = image.height * 3
		
                # create temp path at 45 degrees
                new_vector_name = "temporary path"
                path_id = pdb.gimp_vectors_new(image, new_vector_name)
                pdb.gimp_image_insert_vectors(image, path_id, None, -1)
                points1 = [path_x0,path_y0] * 3
                points2 = [path_x1,path_y1] * 3
                points = points1 + points2
                path_id = pdb.gimp_vectors_stroke_new_from_points(path_id ,0, len(points), points, False)
			
                # Get the current active path i.e. the newly created temp path
                active_vector = pdb.gimp_image_get_active_vectors(image)
                active_vector_name = pdb.gimp_item_get_name(active_vector)
	
                active_vectors = pdb.gimp_image_get_active_vectors(image)
                # Get path stroke information
                NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(active_vector)
                ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
                # set variable to reference the maximum length of the path across the diagonal and remove the temporary path
                max_path_len = int(pdb.gimp_vectors_stroke_get_length(active_vectors,ActivePath_StrokeID,0.01))
                pdb.gimp_image_remove_vectors(image, active_vector)
					
                #for Diagonal text
                vertical = True
                # create random path length
                min_path_len = max_path_len/4 # set a minimum length for the path
                if min_path_len<100: min_path_len = 100 # no less than this
                random.seed()
                path_length = random.randrange(min_path_len, max_path_len) # set the random length for the path
                rotate_text = True
                user_angle = 0
                #pdb.gimp_message("max_path_len = " + str(max_path_len) + " path_length = " + str(path_length) + "\nmin_pos = " + str(min_pos) + "max_pos = " + str(max_pos)  + "\npath_offset_x = " + str(path_offset_x) + "path_offset_y = " + str(path_offset_y) + "\nimage.width - path_length = " + str(image.width - path_length) + "image.height - path_length = " + str(image.height - path_length))
                #pdb.gimp_message("path_x0 = " + str(path_x0) +  " : path_y0 = "  + str(path_y0) +  " : path_x1 = "  + str(path_x1) +  " : path_y1 = "  + str(path_y1))
			
            # draw temporary path	
            if force_it:
                path_length = image.width - 80
                if placement_style == 0:
                    path_x0,path_y0 = 40 ,image.height/2
                    path_x1, path_y1 = 40 + path_length, image.height/2
                else:
                    if placement_style == 1:
                        path_x0,path_y0 = image.width/2,40
                        path_x1, path_y1 = image.width/2, path_length + 40
                    if placement_style == 2:
                        path_x0,path_y0 = 80,80
                        path_x1, path_y1 =  path_length + 80, path_length + 80
	
            new_vector_name = "temporary path"
            path_id = pdb.gimp_vectors_new(image, new_vector_name)
            pdb.gimp_image_insert_vectors(image, path_id, None, -1)
            points1 = [path_x0,path_y0] * 3
            points2 = [path_x1,path_y1] * 3
            points = points1 + points2
            stroke_id = pdb.gimp_vectors_stroke_new_from_points(path_id ,0, len(points), points, False)
		
    
            f_size = 20
	   
            if use_random_gradient_colour:
                random.seed()
                col_loop = random.randrange(0, num_repeats)
                font_colour = fn_shade_gradient_colour(colours_of_samples,col_loop)			
        
            if use_gradient_colouring:
                font_colour = fn_shade_gradient_colour(colours_of_samples,loop)
			
            if use_bg_colour:
                font_colour = blend_colour			
			
            if use_fg_colour:
                font_colour = fg_colour
            final_temp_text = text.rstrip()
            custom_scale = proc_text_along_path(image, layer, final_temp_text, text, font_name_pointer,
                                current_font_name, font_colour, blend_colour, spread_colour, gradient_colour, use_gradient_colour, font_size, step_size, kern_amount, 
                                keep_layers, keep_paths, use_space_to_change_step_size, 
                                compensate_multiplier, use_text_size_modifier, initial_text_size, medial_text_size, final_text_size,  do_best_size,
                                position_of_medial, final_temp_text, do_repeat, remaining_path, reversed_writing, Left_to_Right_path, rotate_text, user_angle, 
                                jitter, jitter_range, tranparency, use_custom_font, xcffilename, fontimg, reversed_vector, custom_font_mode, path_length, vertical, 
                                folder_info, vertical_adjustment_list, horizontal_padding, vertical_padding, diagonal)
        
            path_id = pdb.gimp_image_get_vectors_by_name(image, "temporary path")
            pdb.gimp_image_remove_vectors(image, path_id)
		
            active_layer = pdb.gimp_image_get_active_layer(image)
            if add_drop: proc_dropshadow(image, active_layer, drop_offset, drop_blur)
            if placement_style == 2 and not force_it: # Diagonal Text - move the text from its default diagonal position
                active_layer = pdb.gimp_image_get_active_layer(image)
                active_layer_offset_x,active_layer_offset_y = 0,0
				
					
                if allow_before:
                    val_1 = int(0 - (image.width/1.5))
                    val_2 = int(image.width - (image.width/2))
                    random.seed()
                    active_layer_offset_x = random.randrange(val_1,val_2)
                    val_3 = int(0 - (image.height/1.5))
                    val_4 = int(image.height-(image.height/2))
                    random.seed()
                    active_layer_offset_y = random.randrange(val_3,val_4)
                    pdb.gimp_layer_set_offsets(active_layer, active_layer_offset_x, active_layer_offset_y)
					
                elif allow_after:
                    val_1 = int(0 - (image.width/10))
                    val_2 = int(image.width - (image.width/1.5))
                    random.seed()
                    active_layer_offset_x = random.randrange(val_1,val_2)
                    val_3 = int(0 - (image.height/10))
                    val_4 = int(image.height-(image.height/1.5))
                    random.seed()
                    active_layer_offset_y = random.randrange(val_3,val_4)



                    pdb.gimp_layer_set_offsets(active_layer, active_layer_offset_x, active_layer_offset_y)
                else:
                
                    random.seed()
                    w_factor = (random.randrange(10000000)%2)+1
                    offset_xcheck = int((image.width/w_factor) - (vertical_adjustment_list[6] * custom_scale)) # width - width of 1st character
                    if offset_xcheck<10:offset_xcheck = 10
                    random.seed()
                    active_layer_offset_x = random.randrange(0, offset_xcheck)
                    offset_ycheck = int(image.height - (vertical_adjustment_list[7] * custom_scale ))# height - height of 1st character
                    if offset_ycheck<10:offset_ycheck = 10
                    random.seed()
                    active_layer_offset_y = random.randrange(0, offset_ycheck)
                    pdb.gimp_layer_set_offsets(active_layer, active_layer_offset_x, active_layer_offset_y)
		
            loop += 1

    #if my_error<>"": pdb.gimp_message(my_error)
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
		
######################################################################################################################################

def proc_dropshadow(im, target_layer, drop_offset, drop_blur):
    #REM Create a text drop shadow effect - optionally clipped to the text layer for transparent type texts
    old_im_width, old_im_height = im.width, im.height
    max_size = max(target_layer.width,target_layer.height)
    size_adj = (max_size/100)*5
    sh_blur = size_adj
    sh_xpos, sh_ypos = size_adj,size_adj
    name = pdb.gimp_item_get_name(target_layer)
    #REM: Copy layer
    shadow_layer = pdb.gimp_layer_copy(target_layer, False)
    im.add_layer(shadow_layer, -1)
    pdb.gimp_image_lower_item(im, shadow_layer)
    pdb.gimp_layer_resize(shadow_layer, target_layer.width + size_adj, target_layer.height + size_adj, size_adj, size_adj)
	
    offset_x, offset_y = pdb.gimp_drawable_offsets(shadow_layer)
    if shadow_layer.width + offset_x + size_adj > im.width or shadow_layer.height + offset_y + size_adj > im.height:
        pdb.gimp_image_resize(im, im.width + size_adj, im.height + size_adj, 0, 0)
		
    pdb.gimp_image_select_item(im, 2, shadow_layer)
    pdb.gimp_drawable_edit_fill(shadow_layer,1)
    pdb.gimp_selection_none(im)
    pdb.plug_in_gauss(im, shadow_layer, drop_blur*5, drop_blur*5, 1)
    temp = pdb.gimp_layer_translate(shadow_layer, drop_offset, drop_offset)
    pdb.gimp_layer_set_opacity(shadow_layer, 70)
    target_layer = pdb.gimp_image_merge_down(im,target_layer,0)
    pdb.gimp_item_set_name(target_layer, name)
    pdb.gimp_image_resize(im, old_im_width, old_im_height, 0, 0)
	
######################################################################################################################################
def fn_find_layer_in_list(vertical_adjustment_list, custom_layer_name):
    # finds the position of a character in the list in order to reference x0, y0 etc of character within the layer
    loop, pos, alt_pos = 1, -1, -1
    alt_custom_layer_name = custom_layer_name[0]
    if alt_custom_layer_name.islower:
        alt_custom_layer_name = alt_custom_layer_name.upper()
    elif  alt_custom_layer_name.isupper:
        alt_custom_layer_name = alt_custom_layer_name.lower()
    alt_custom_layer_name = alt_custom_layer_name + ".png"
		
    while loop<len(vertical_adjustment_list):
        if custom_layer_name == vertical_adjustment_list[loop]: 
            pos = loop
            loop = len(vertical_adjustment_list)+1
        loop += 7
		
    if pos == -1:
        alt_pos, loop = 1,0
        while loop<len(vertical_adjustment_list):
            if alt_custom_layer_name == vertical_adjustment_list[loop]: 
                alt_pos = loop
                loop = len(vertical_adjustment_list)+1
            loop += 7
        pos = alt_pos
	
    return pos
	
        
######################################################################################################################################

def fn_get_tight_character_ht_info(image, chara, fontimg):
    if chara == " ":
        fontimg_layer = pdb.gimp_image_get_layer_by_name(fontimg, "space" + ".png")
        cs_x1, cs_y1 = 0, 0
        cs_y2, ht = fontimg_layer.height/2, fontimg_layer.height/2
        cs_x2, wd =  fontimg_layer.width/2, fontimg_layer.width/2
    else:
        fontimg_layer = pdb.gimp_image_get_layer_by_name(fontimg, chara + ".png")
        if fontimg_layer<0:
            if chara.islower():
                chara = chara.upper() 
                fontimg_layer = pdb.gimp_image_get_layer_by_name(fontimg, chara + ".png")
        copy_layer = pdb.gimp_layer_new_from_drawable(fontimg_layer,image)
        temp = pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
        pdb.gimp_item_set_visible(copy_layer, True)
        pdb.gimp_layer_set_offsets(copy_layer,0,0)
        copy_layer2 = pdb.gimp_layer_new_from_drawable(fontimg_layer,image)
        temp2 = pdb.gimp_image_insert_layer(image, copy_layer2, None, -1)
        pdb.gimp_item_set_visible(copy_layer2, True)
        pdb.gimp_layer_set_offsets(copy_layer2,0,0)
        ht = copy_layer.height
        wd = copy_layer.width
        pdb.plug_in_autocrop_layer(image, copy_layer)
        pdb.gimp_drawable_fill(copy_layer2, 1)
        copy_layer = pdb.gimp_image_merge_down(image,copy_layer2,0)
        pdb.gimp_image_select_item(image, 2, copy_layer)
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2  = pdb.gimp_selection_bounds(image)
        pdb.gimp_selection_none(image)
        image.remove_layer(copy_layer)
    return cs_x1, cs_y1, cs_x2, cs_y2, wd, ht

######################################################################################################################################

def fn_adjusted_char_height(image, layer):
    copy_layer = pdb.gimp_layer_new_from_drawable(layer,image)
    temp = pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
    pdb.gimp_item_set_visible(copy_layer, True)
    pdb.gimp_layer_set_offsets(copy_layer,0,0)
    copy_layer2 = pdb.gimp_layer_new_from_drawable(layer,image)
    temp2 = pdb.gimp_image_insert_layer(image, copy_layer2, None, -1)
    pdb.gimp_item_set_visible(copy_layer2, True)
    pdb.gimp_layer_set_offsets(copy_layer2,0,0)
    pdb.plug_in_autocrop_layer(image, copy_layer)
    pdb.gimp_drawable_fill(copy_layer2, 1)
    copy_layer = pdb.gimp_image_merge_down(image,copy_layer2,0)
    pdb.gimp_image_select_item(image, 2, copy_layer)
    current_selection, cs_x1, cs_y1, cs_x2, cs_y2  = pdb.gimp_selection_bounds(image)
    pdb.gimp_selection_none(image)
    image.remove_layer(copy_layer)
    adjustment = cs_y1 + (cs_y2 - cs_y1)
    return adjustment

######################################################################################################################################

def fn_test_for_custom_chars(image, text):
    # checks that the custom fonts contain the character layers
    chars_in_ui_text = fn_chars_in_text(text)
    test = True
    if " " not in chars_in_ui_text: chars_in_ui_text += " "
    for char in chars_in_ui_text:
        if char == " ": char = "space"
        current_id_ucase = pdb.gimp_image_get_layer_by_name(image, char.upper() + ".png")
        current_id_lcase = pdb.gimp_image_get_layer_by_name(image, char.lower() + ".png")
        if (current_id_ucase < 0 and current_id_lcase < 0): test = False
    return test
	
#######################################################################################################

def fn_chars_in_text(text):
    loop = 0
    while loop < len(text):
        if loop == 0:
            new_text = text[loop]
        else:
            if text[loop] not in new_text: new_text += text[loop]
        loop += 1
    return new_text 
        
#######################################################################################################

def fn_shade_gradient_colour(colours_of_samples, instances):
    red_amount = colours_of_samples[(instances*4) + 0] * 255
    green_amount = colours_of_samples[(instances*4) + 1] * 255
    blue_amount = colours_of_samples[(instances*4) + 2] * 255
    alpha_amount = colours_of_samples[(instances*4) + 3] * 255

    if red_amount < 0: red_amount = 0
    if red_amount > 255: red_amount = 255
    if green_amount < 0: green_amount = 0
    if green_amount > 255: green_amount = 255
    if blue_amount < 0: blue_amount = 0
    if blue_amount > 255: blue_amount = 255
    if alpha_amount < 0: alpha_amount = 0
    if alpha_amount > 255: alpha_amount = 255
	
    new_f_colour = (int(red_amount), int(green_amount), int(blue_amount), int(alpha_amount))
    return new_f_colour
	
#######################################################################################################

def fn_get_gradient_positions(instances):
    # get positions along gradients for text colours
    grad_position = 0.0
    if instances<1: instances = 1
    grad_amount = float(1.00/instances)
    loop = 0
    while loop<=instances:
        if loop == 0:
            list_of_pos = [grad_position]
        else:
            list_of_pos.append(grad_position)
        grad_position += grad_amount
        loop += 1
    return list_of_pos


#######################################################################################################

def fn_font_x_pos_adjustment(kerning, f_size):
    adjust_factor = (((100 * kerning) / 1024) * f_size) / 100
    return adjust_factor

#######################################################################################################
	
def fn_compensate_for_curve_in_path(p_angle, c_angle, f_size):
    curve_compensation,angle_diff = 0,0
	
    if (c_angle<0 and p_angle>180):
        c_angle += 360
    if (c_angle>180 and p_angle<0):
        c_angle -= 360
	
    if (c_angle>p_angle): # tests for text along path travelling in a clockwise direction
        
        min_angle = min(abs(p_angle), abs(c_angle))
        max_angle = max(abs(p_angle), abs(c_angle))
        angle_diff = (min_angle - max_angle)
        curve_compensation = fn_font_x_pos_adjustment(angle_diff, f_size)                

    if (c_angle<p_angle): # tests for  text along path travelling in a counter-clockwise direction
						
        min_angle = min(abs(p_angle), abs(c_angle))
        max_angle = max(abs(p_angle), abs(c_angle))
        angle_diff = max_angle - min_angle
        curve_compensation = fn_font_x_pos_adjustment(angle_diff,f_size)
		
    return curve_compensation,angle_diff

#######################################################################################################

def computeThetaWithSlope(fromPoint,toPoint,slope):
    dX=toPoint[0]-fromPoint[0]
    dY=toPoint[1]-fromPoint[1]

    if math.fabs(dX) < .001:
        theta=math.copysign(math.pi/2,dY)
    else:
        theta=math.atan(slope)
        if dX<0:
            theta=theta+math.pi
    return theta
	
#######################################################################################################

def fn_label_characters(label_text,label_len):
    label_char_info = label_text
    label_len_info = len(label_text)
    label_etc = ""
    if label_len_info > label_len: 
        label_char_info = label_char_info[0:label_len]
        if label_char_info.count(" ")>0:
            loop = 0
            space_pos = label_len - 1
            while loop<label_len:
                if label_char_info[loop] == " ": space_pos = loop
                loop += 1
            label_char_info = label_char_info[0:space_pos]
            label_etc = " etc"
    if label_len_info == 1: 
        label_len_detail = " char" 
    else:
        label_len_detail = " chars" 
    details = label_char_info + label_etc + " " + str(label_len_info ) + label_len_detail
    return details
	
###########################################################################################

def fn_size_to_fit_path(image, text, f_name, f_size, kern_amount, use_custom, xcffilename, fontimg, path_length, vertical, vertical_adjustment_list, horizontal_padding, vertical_padding, diagonal):

    # Get the current active path
	
    # Set up some variables
    if path_length<50:
        f_size_seed = 10
    elif path_length<100:
        f_size_seed = 50
    elif path_length<500:
        f_size_seed = 100
    elif path_length<1000:
        f_size_seed = 500
    else:
        f_size_seed = 1000
		
		
    font_name_pointer = -1
    f_size = f_size_seed
    char_width, NextChr_width = 0,0
    kerning_flag = -1
    # Calculate kerning to be used between characters, if required by user, based on size of font taken from user settings
    kern_gap = kern_amount
    tempbox_width = 0
    loop = 0
    padding = 0
    f_xpos = 0
    init_f_xpos = f_xpos
    scaled_xpos = 0
    curve = 0
	
    space_name = " .png"
    char_name = " "
    layer_name = char_name + ".png"
	
	
    if use_custom:
        char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
        font_name_pointer = -1
        space_has_vectors = False
        if char_layer < 0: #layer not found
            space_name = "space.png"
            char_name = "space"
            layer_name = char_name + ".png"
            char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
        if char_layer < 0: #layer not found
            space_name = "nospace"
            #pdb.gimp_message("No space character..." )
            space_width = f_size * 1.164 # Make up a space width for this font size
            space_height = f_size
            font_y_offset = (f_size * 1.164)/2
        else:
            custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
            space_width = char_layer.width * custom_scale
            space_height = char_layer.height * custom_scale
            font_y_offset = space_height/2
    else:
	
        # Find the width to use for the space character
        (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(" ", f_size, PIXELS, str(f_name))
        space_width, space_height = fontbox_width, fontbox_height
        #### should we check for paths/glyphs in the space character
		
        # Check to see if the space character has paths so that we can set a flag if necessary
        temp_space_string = " -"
        temp_space_layer = pdb.gimp_text_layer_new(image,temp_space_string,f_name,f_size,0)
        pdb.gimp_image_insert_layer(image,temp_space_layer,None,-1)
        temp = pdb.gimp_vectors_new_from_text_layer(image, temp_space_layer)
        space_vectors_path = pdb.gimp_image_insert_vectors(image,temp,None,-1)
        space_vectors = pdb.gimp_image_get_vectors_by_name(image, temp_space_string)
        # Get space character path stroke information
        NumberOfStrokes_in_Space_Path,Space_Path_StrokeIDs_List = pdb.gimp_vectors_get_strokes(space_vectors)
        pdb.gimp_image_remove_vectors(image,space_vectors)
        temp = pdb.gimp_image_remove_layer(image, temp_space_layer)

        # Check to see if the font uses a (visible) glyph in the space character to set a flag
        space_has_vectors = False
        if (NumberOfStrokes_in_Space_Path > 1):
            if (font_name_pointer > -1): space_has_vectors = True
			
    if not vertical: # set value for user adjustments to character spacing
        kern_gap = ((float(space_width)/100))*horizontal_padding
        #if kern_gap < 0: kern_gap = 0
    else:
        kern_gap = ((float(space_height)/100))*vertical_padding
        #if kern_gap < 0: kern_gap = 0
    
    while loop < len(text):
        #### put check for space character here????
		
        CurrentChar = text[loop]
        if use_custom:
            ch_layer = pdb.gimp_image_get_layer_by_name(fontimg,space_name)
            ch_scale = fn_get_custom_font_scale(f_size, ch_layer.height)
            sp_width = ch_layer.width * ch_scale
			
        # Find the width to use for the space character
        if CurrentChar == " ":
            if use_custom:
                char_layer = pdb.gimp_image_get_layer_by_name(fontimg,space_name)
                custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
                space_width = char_layer.width * custom_scale
                custom_width = space_width
                scaled_width = space_width
            else:
                (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(" ", f_size, PIXELS, str(f_name))
                space_width = fontbox_width
                fontbox_width = space_width
                scaled_width = space_width
		
        if loop < (len(text)-1):
            NextChar = text[loop + 1]
        else:
            NextChar = ""
			
        # reset character x position adjustment factors
        xpos_adjustment = 0
		
        if (CurrentChar > " " or (CurrentChar == " " and space_has_vectors == True)):
            custom_layer_name = CurrentChar + ".png"
            # Create layers for each character in user defined string	

            if use_custom:
                # get id of character from custom font file in memory
                fontimg_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                if fontimg_layer < 0: #if letter not found try swapping upper/lower case to look for
                    char = CurrentChar
                    if char.lower() == char:
                        char = char.upper()
                    else:
                        char = char.lower()
                    custom_layer_name = char + ".png"
                    fontimg_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                if NextChar<>"": 
                    fontimg_layer2 = pdb.gimp_image_get_layer_by_name(fontimg, NextChar+".png")
                    if fontimg_layer2 < 0: #if letter not found try swapping upper/lower case to look for
                        nchar = NextChar
                        if nchar.lower() == nchar:
                            nchar = nchar.upper()
                        else:
                            nchar = nchar.lower()
                    fontimg_layer2 = pdb.gimp_image_get_layer_by_name(fontimg, NextChar+".png")
                ####
                # sort out scaling to get the font size we need
                custom_scale = fn_get_custom_font_scale(f_size, fontimg_layer.height)
                custom_width = (fontimg_layer.width  + kern_gap) * custom_scale
                custom_height = fontimg_layer.height * custom_scale
				
								
                if vertical: ################# added for loads of text
                    find_me = fn_find_layer_in_list(vertical_adjustment_list,CurrentChar + ".png")
                    if NextChar<>"":
                        find_me_next_layer = pdb.gimp_image_get_layer_by_name(fontimg, NextChar + ".png")
                        if find_me_next_layer<0:
                            if NextChar.islower():
                                find_me_next_layer = pdb.gimp_image_get_layer_by_name(fontimg, NextChar.upper() + ".png")
                                find_me_next = fn_find_layer_in_list(vertical_adjustment_list,NextChar.upper()+".png")
                        else:
                            find_me_next = fn_find_layer_in_list(vertical_adjustment_list,NextChar+".png")
                    height_adjusted = (vertical_adjustment_list[find_me + 4])
                    if NextChar<>"": 
                        temp_y = vertical_adjustment_list[find_me_next + 2]
                        height_adjusted += (0-temp_y)
                    custom_width = height_adjusted * custom_scale
					
					
                    if diagonal:
                        temp_width = (vertical_adjustment_list[find_me + 5])/2.25
                        if NextChar<>"":
                            NextChar_name = NextChar + ".png"
                            temp_width2 = (vertical_adjustment_list[find_me_next + 5])/2.25
                            custom_width = ((temp_width)+(temp_width2))/(math.sin(0.78534031413613)) * custom_scale
                        else:
                            custom_width = ((temp_width*2)/(math.sin(0.78534031413613))) * custom_scale
				
                scaled_width = custom_width

                ####
                # create, insert and set visibility the copy of the character layer into our image
            else:
                #### New 0.10 lines
                font_scale = fn_get_custom_font_scale(f_size, fontbox_height)
                char_layer = pdb.gimp_text_layer_new(image,CurrentChar,f_name,f_size,0)
				
                fontbox_width = char_layer.width #+ (kern_gap * font_scale)
                scaled_width = char_layer.width
				
				
	        #### Look for kerning path information
            kerning_vector = -1
            char_vector = -1
            if use_custom:
                char_vector = pdb.gimp_image_get_vectors_by_name(fontimg,custom_layer_name)
                kerning_name, kerning_name_2 = CurrentChar + NextChar, CurrentChar + NextChar + ".png"
                kerning_vector = pdb.gimp_image_get_vectors_by_name(fontimg,kerning_name) # check for character kerning pair paths without .png
                if kerning_vector == None: 
                    kerning_vector = pdb.gimp_image_get_vectors_by_name(fontimg,kerning_name_2) # check for character kerning pair paths with .png
                    
                if char_vector>-1 or kerning_vector>-1: kerning_flag = 0                
					
            if char_vector >=0 and kerning_vector <0:
                # Get path info for current character path to work out character width
                char_vector_name = pdb.gimp_item_get_name(char_vector)
                NumberOfStrokes_in_Path,Char_Vector_StrokeIDs_List = pdb.gimp_vectors_get_strokes(char_vector)
                Char_Vector_Stroke = Char_Vector_StrokeIDs_List[0]
                TypeOfStrokeInChar_Vector,NumberOfPointsInChar_Vector,ListOfControlPoints,IsStrokeClosed = pdb.gimp_vectors_stroke_get_points(char_vector,Char_Vector_Stroke)
                firstX = ListOfControlPoints[2]
                secondX = ListOfControlPoints[len(ListOfControlPoints)-4]
                minX = min(firstX,secondX)
                maxX = max(firstX,secondX)
                vector_char_width = maxX - minX #overwrite char_width to be used as character width.
                add_width = vector_char_width
                if use_custom and not vertical: custom_width = add_width * custom_scale ##################### altered for loads of text
                if use_custom and not vertical: f_xpos += (minX * custom_scale) # Slab v 0.06 caters for negative kerning with custom fonts ##################### altered for loads of text
				
				
            if kerning_vector >= 0: 
                NumberOfStrokes_in_Path,Char_Vector_StrokeIDs_List = pdb.gimp_vectors_get_strokes(kerning_vector)
                Char_Vector_Stroke = Char_Vector_StrokeIDs_List[0]
                TypeOfStrokeInChar_Vector,NumberOfPointsInChar_Vector,ListOfControlPoints,IsStrokeClosed = pdb.gimp_vectors_stroke_get_points(kerning_vector,Char_Vector_Stroke)
                firstX = ListOfControlPoints[2]
                secondX = ListOfControlPoints[len(ListOfControlPoints)-4]
                minX = min(firstX,secondX)
                maxX = max(firstX,secondX)
                kerning_width = maxX - minX #overwrite char_width to be used as character width.
                add_width = kerning_width
                if use_custom and not vertical: custom_width = add_width * custom_scale ##################### altered for loads of text
                if use_custom and not vertical: f_xpos += (minX * custom_scale) # Slab v 0.06 caters for negative kerning with custom fonts ##################### altered for loads of text
				
        #### Update f_xpos to take width of text along path from
        if use_custom:
            if loop == len(text)-1 and kerning_flag == 0: 
                if diagonal:
                    custom_width = custom_width + (custom_width * 0.5)
                    scaled_width = scaled_width + (scaled_width * 0.5)
                else:
                    custom_width = custom_width + (custom_width * 0.5)
                    scaled_width = scaled_width + (scaled_width * 0.5)
					
					
                if vertical: ################# added for loads of text
                    dummy = 0
                    find_me = fn_find_layer_in_list(vertical_adjustment_list,custom_layer_name)
                    if NextChar<>"": 
                        find_me_next = fn_find_layer_in_list(vertical_adjustment_list,NextChar+".png")
                    else:
                        find_me_next = fn_find_layer_in_list(vertical_adjustment_list,CurrentChar+".png")
                    height_adjusted = (vertical_adjustment_list[find_me + 4]) + (vertical_adjustment_list[find_me + 4] * 0.5)
                    height_adjusted += (0-vertical_adjustment_list[find_me_next + 2])
                    custom_width = height_adjusted * custom_scale
                    scaled_width = custom_width
					
                    if diagonal:
                        temp_width = (vertical_adjustment_list[find_me + 5])/2.25
                        if NextChar<>"":
                            NextChar_name = NextChar + ".png"
                            temp_width2 = (vertical_adjustment_list[find_me_next + 5])/2.25
                            custom_width = ((temp_width)+(temp_width2))/(math.sin(0.78534031413613)) * custom_scale
                        else:
                            custom_width = ((temp_width*2)/(math.sin(0.78534031413613))) * custom_scale
				
                        scaled_width = custom_width
            scaled_xpos = scaled_xpos + scaled_width + curve - xpos_adjustment
            #f_xpos = f_xpos + custom_width + kern_gap + curve - xpos_adjustment	
            f_xpos = f_xpos + custom_width + curve - xpos_adjustment	
            if not loop == len(text)-1: f_xpos = f_xpos + abs(kern_gap)
            #if f_xpos<0: f_xpos = 0
            if f_xpos < init_f_xpos: f_xpos = init_f_xpos
        else:
            if loop == len(text)-1: fontbox_width = fontbox_width + fontbox_width * 0.5
            f_xpos = f_xpos + fontbox_width + kern_gap + curve - xpos_adjustment
            if loop == len(text)-1: 
                    scaled_width = scaled_width + (scaled_width * 0.5)
            scaled_xpos = scaled_xpos + scaled_width + curve - xpos_adjustment
        loop += 1
    original_f_xpos = f_xpos
    f_size = ((path_length/f_xpos)*f_size_seed)
    padding = int((path_length - ((f_xpos/f_size_seed)*f_size))/2)
    if padding<0: padding = 0
    if use_custom:
        padding += int(((scaled_width/f_size_seed)*f_size)*0.125)
    else:
        padding += int(((scaled_width/f_size_seed)*f_size)*0.2)
    if f_size>3000: f_size = 3000
    if f_size<2: f_size = 2
    
	
    return f_size, padding, f_xpos, path_length, f_size_seed
################################################################  ######################################################################
    
def proc_text_along_path(image, layer, joining_text_list, text, font_name_pointer, 
                        f_name, f_colour,  bl_colour, spread_colour, joinedup_gradient, use_gradient_colour, f_size, step_size, kern_amount, 
                        keep_layers, keep_paths, use_space_to_change_step_size, compensate_multiplier, 
                        use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, do_best_size,
                        position_of_medial, final_temp_text, repeat_text, remaining_path, reversed_writing, Left_to_Right_path, rotate_text, 
                        user_angle, jitter, jitter_range, tranparency, use_custom, xcffilename, fontimg, reversed_vector, custom_font_mode, path_length, 
                        vertical, folder_info,vertical_adjustment_list, horizontal_padding, vertical_padding, diagonal):

				
    # Creates text layers
	
	#### New function to calculate font size that will fit the length of the path
    offset_x = 0
    custom_scale = 1
    if do_best_size: f_size, path_padding, best_fit_f_xpos, bf_path_length, f_seed = fn_size_to_fit_path(image, text, f_name, f_size, kern_amount, use_custom, xcffilename, fontimg, path_length, vertical, vertical_adjustment_list, horizontal_padding, vertical_padding, diagonal)
    
    initial_step_size = step_size
    label_details = fn_label_characters(text, 12)
    label_fontsize_info = " " + str(int(f_size)) + "pt"
    label_fontname = ""
    label_rotate = ""
    label_jitter = ""
    label_reverse = ""
    if use_custom:
        label_fontname = fn_get_custom_font_name(xcffilename)
        label_fontname = " Custom Font " + label_fontname
    else:
        label_fontname = f_name
    
    original_user_angle = user_angle
    governor = 0 # initialise variable to use to prevent creation of unnecessary layers
    attack, decay = 0, 0 # initialise variable to mark point of change in text size within text string when size modifier flag is set
    spaces_loop = 0
    spaces_count = text.count(" ")
	
    if spaces_count > 0:
		
        if text.endswith(" "):
            padding = remaining_path / (spaces_count + 0)
        else:
            padding = remaining_path / (spaces_count + 1)
            
    else:
        if font_name_pointer > -1:
            padding = 0
        else:
            if text.endswith(" "):
                padding = remaining_path / len(joining_text_list)
            else:
                padding = remaining_path / (len(joining_text_list) + 1)
						
    if not use_gradient_colour:
        if spaces_count<2: spaces_count = 2
    target = 0
    original_font_colour = f_colour
    shading_loop = 0
	
    if len(text) % 2 == 0: # variable to help position attack (first) phase of change in text size
        even = True
    else:
        even = False
		
    if " " not in text: use_space_to_change_step_size = False # Force option off, if there are no space characters in the text

    if use_gradient_colour: 
        # get number of colours etc needed for using colours from current ui gradient
        current_gradient = joinedup_gradient
        list_of_gradient_positions = fn_get_gradient_positions(len(final_temp_text))
        number_of_samples, colours_of_samples = pdb.gimp_gradient_get_custom_samples(current_gradient, len(list_of_gradient_positions), list_of_gradient_positions,False)
    # Get the current active path
    active_vector = pdb.gimp_image_get_active_vectors(image)
    active_vector_name = pdb.gimp_item_get_name(active_vector)
	
    if (active_vector == -1):
        active_vector = pdb.gimp_image_set_active_vectors(image,image.vectors[0])
	
    active_vectors = pdb.gimp_image_get_active_vectors(image)
    # Get path stroke information
    NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(active_vector)
    ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
	
	# initialise some variables
    temp_f_xpos = path_padding + 0
    f_xpos = path_padding + 0
    init_f_xpos = f_xpos
		
    f_ypos = 40
    curve = 0
    curve_compensation = 0
    temp_angle_diff = 0
	
    if use_custom:
        
        file_dummy = 0
        space_name = " .png"
        char_name = " "
        layer_name = char_name + ".png"
        char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
        if char_layer < 0: #layer not found
            space_name = "space.png"
            char_name = "space"
            layer_name = char_name + ".png"
            char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
        if char_layer < 0: #layer not found
            space_name = "nospace"
            #pdb.gimp_message("No space character..." )
            space_width = f_size * 1.164 # Make up a space width for this font size
            space_height = f_size
        else:
            custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
            space_width = char_layer.width * custom_scale
            space_height = char_layer.height * custom_scale
            font_y_offset = (char_layer.height * 1.164)/2
			
        if not vertical:
            kern_gap = ((float(space_width)/100))*horizontal_padding
        else:
            
            kern_gap = ((float(space_height)/100))*vertical_padding
        
    else:
	
        # Find the width to use for the space character
        (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(" ", f_size, PIXELS, str(f_name))
        space_width = fontbox_width
        space_height = fontbox_height
		
        if not vertical:
            kern_gap = ((float(space_width)/100))*horizontal_padding
        else:
            kern_gap = ((float(space_height)/100))*vertical_padding
	
        # Check to see if the space character has paths so that we can set a flag if necessary
        temp_space_string = " -"
        temp_space_layer = pdb.gimp_text_layer_new(image,temp_space_string,f_name,f_size,0)
        pdb.gimp_image_insert_layer(image,temp_space_layer,None,-1)
        temp = pdb.gimp_vectors_new_from_text_layer(image, temp_space_layer)
        space_vectors_path = pdb.gimp_image_insert_vectors(image,temp,None,-1)
        space_vectors = pdb.gimp_image_get_vectors_by_name(image, temp_space_string)
        # Get space character path stroke information
        NumberOfStrokes_in_Space_Path,Space_Path_StrokeIDs_List = pdb.gimp_vectors_get_strokes(space_vectors)
        pdb.gimp_image_remove_vectors(image,space_vectors)
        temp = pdb.gimp_image_remove_layer(image, temp_space_layer)

        # Check to see if the font uses a (visible) glyph in the space character to set a flag
        space_has_vectors = False
        if (NumberOfStrokes_in_Space_Path > 1):
            if (font_name_pointer > -1): space_has_vectors = True

    text_layer_y_origin = image.height - (f_size + f_ypos)
			
    initial_f_size = f_size		
    if initial_f_size < 2: initial_f_size = 2
    if initial_f_size > 3000: initial_f_size = 3000
    
    if not use_custom:
        (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname("dummy", initial_f_size, PIXELS, str(f_name))
        font_y_offset = fontbox_ascent

    char_width, NextChr_width = 0,0
    # Calculate kerning to be used between characters, if required by user, based on size of font taken from user settings
    kern_factor = kern_amount/f_size
    tempbox_width = 0
    loop = 0
	
    if use_custom: 
        font_name_pointer = -1
        space_has_vectors = False
	
    # Initialise a loop specifically to use for creating layer and path identifiers and names
    layer_and_path_loop = 0
    # Create a list to hold the text layer identifiers
    layer_id_list = ["dummy"]
    # Create a list to hold the layer name
    layer_name_list = ["dummy"]
	
    if not use_custom: 
        path_id_list = ["dummy"]
        paths_names_list = ["dummy"]
	
	
			        
    while loop < len(joining_text_list) and (f_xpos + (tempbox_width )< path_length):
        CurrentChar, RealCurrentChar = joining_text_list[loop], text[loop]
        tempbox_width = 0

        if loop > 0:		        
            if " " not in joining_text_list:
                f_xpos += padding
            else:
                # Find the width to use for the space character
                if CurrentChar == " ":
                    if use_custom:
                        char_layer = pdb.gimp_image_get_layer_by_name(fontimg,space_name)
                        custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
                        space_width = char_layer.width * custom_scale
                        custom_width = space_width
                    else:
                        (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(" ", f_size, PIXELS, str(f_name))
                        space_width = fontbox_width
                    f_xpos += padding
        if loop < (len(joining_text_list)-1):
            NextChar, RealNextChar = joining_text_list[loop + 1], text[loop + 1]
        else:
            NextChar, RealNextChar = "", ""
			
        # reset character x position adjustment factors
        f_xpos_adjustment, j_xpos_adjustment, p_xpos_adjustment, t_xpos_adjustment = 0,0,0,0
        xpos_adjustment = 0

        if (font_name_pointer > -1):
            # Adjust kerning
            xpos_adjustment += fn_adjust_xpos(NextChar, CurrentChar, RealCurrentChar, RealNextChar, font_name_pointer, f_size, joining_text_list[loop])
				
        if (CurrentChar > " " or (CurrentChar == " " and space_has_vectors == True)):
            custom_layer_name = CurrentChar + ".png"
            # Create layers for each character in user defined string	
			
            if use_custom:
                # get id of character from custom font file in memory
                char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                char_vector = pdb.gimp_image_get_vectors_by_name(fontimg,custom_layer_name)
                if char_layer < 0: #if letter not found try swapping upper/lower case to look for
                    char = CurrentChar
                    if char.lower() == char:
                         char = char.upper()
                    else:
                        char = char.lower()
                    custom_layer_name = char + ".png"
                    char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                if NextChar<>"": 
                    fontimg_layer2 = pdb.gimp_image_get_layer_by_name(fontimg, NextChar+".png")
                    if fontimg_layer2 < 0: #if letter not found try swapping upper/lower case to look for
                        nchar = NextChar
                        if nchar.lower() == nchar:
                            nchar = nchar.upper()
                        else:
                            nchar = nchar.lower()
                    fontimg_layer2 = pdb.gimp_image_get_layer_by_name(fontimg, NextChar+".png")
                # work out scale of character 
                custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
                # create, insert and set visibility the copy of the character layer into our image
                copy = pdb.gimp_layer_new_from_drawable(char_layer,image)
                pdb.gimp_image_insert_layer(image,copy,None,0)
                pdb.gimp_item_set_visible(copy,True)
                # sort out scaling to get the font size we need
                custom_width = copy.width * custom_scale
                custom_height = copy.height * custom_scale
				
                old_interpolation = pdb.gimp_context_get_interpolation() # Note current interpolation method
                pdb.gimp_context_set_interpolation(3) # Ensure that interpolation is set to Sinc (Lancos 3) method
                pdb.gimp_layer_scale(copy,custom_width,custom_height,False)
                pdb.gimp_context_set_interpolation(old_interpolation) # Restore old interpolation method
                pdb.gimp_layer_set_offsets(copy,0,0)
					
                # Add further layer identifiers to the list of text layer identifiers
                layer_id_list.append(copy)
                custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
                char_width = custom_width
				
				
				
                if vertical: ################# added for loads of text
						
                    find_me = fn_find_layer_in_list(vertical_adjustment_list,custom_layer_name) # pointer to character in list
                    ht_adj_part_1 = vertical_adjustment_list[find_me + 4] # stored y1 value for current character in list
                    if NextChar<>"":
                        find_me_next = fn_find_layer_in_list(vertical_adjustment_list,NextChar+".png") # if a next character exists get pointer to character in list
                        ht_adj_part_2 = vertical_adjustment_list[find_me_next + 2]
                    else:
                        find_me_next = fn_find_layer_in_list(vertical_adjustment_list,CurrentChar+".png") # if no next character exists set the pointer to the current character
                        ht_adj_part_2 = vertical_adjustment_list[find_me_next + 2]
                    height_adjusted = ht_adj_part_1 - ht_adj_part_2 # set the height adjustment to y1 - y0 ie the height of the alpha selection for the character
                    custom_width = height_adjusted * custom_scale
					
					
                    if diagonal:
                        if NextChar<>"":
                            NextChar_name = NextChar + ".png"
                            temp_width = (vertical_adjustment_list[find_me + 5])/2.25
                            temp_width2 = (vertical_adjustment_list[find_me_next + 5])/2.25
                            custom_width = ((temp_width)+(temp_width2))/(math.sin(0.78534031413613)) * custom_scale
                        else:
                            custom_width = (temp_width*2)/(math.sin(0.78534031413613)) * custom_scale
				
						
                # Set the new custom layer colour from the current font colour and the colour to shade to
                if spread_colour and not use_gradient_colour: 
                    if not use_space_to_change_step_size:
                        custom_colour = fn_shade_text_colour(original_font_colour, bl_colour, len(text)-1, loop + 1)
                    else:
                        custom_colour = fn_shade_text_colour(original_font_colour, bl_colour, spaces_count - 1, shading_loop)
                    proc_colour_character(image, layer_id_list[layer_and_path_loop + 1], custom_colour, custom_font_mode)
                # Set the new custom layer colour from the current gradient sampled colour
                if use_gradient_colour:
                    custom_colour = fn_shade_gradient_colour(colours_of_samples,loop + 1)
                    proc_colour_character(image, layer_id_list[layer_and_path_loop + 1], custom_colour, custom_font_mode)
					
            else:
                # Add further layer identifiers to the list of text layer identifiers
                layer_id_list.append(["text_layer" + str(layer_and_path_loop + 1)])
                # and create each subsequent layer
                layer_id_list[layer_and_path_loop + 1] = pdb.gimp_text_layer_new(image,CurrentChar,f_name,f_size,0)
				
                # Insert subsequent layers into the image
                pdb.gimp_image_insert_layer(image,layer_id_list[layer_and_path_loop + 1],None,-1)
				
                # Set the new text layer colour
                if spread_colour and not use_gradient_colour: 
                    if not use_space_to_change_step_size:
                        f_colour = fn_shade_text_colour(original_font_colour, bl_colour, len(text)-1, loop + 1)
                    else:
                        f_colour = fn_shade_text_colour(original_font_colour, bl_colour, spaces_count - 1, shading_loop)
                # Set the new text layer colour from the current gradient sampled colour
                if use_gradient_colour:
                    if_colour = fn_shade_gradient_colour(colours_of_samples,loop + 1)
						
                pdb.gimp_text_layer_set_color(layer_id_list[layer_and_path_loop + 1], f_colour)
				
                # Get width of current character
                char_width = layer_id_list[layer_and_path_loop + 1].width
                offset_x,offset_y = pdb.gimp_drawable_offsets(layer_id_list[layer_and_path_loop+1])
            # Set layer opacity as per user setting
            pdb.gimp_layer_set_opacity(layer_id_list[layer_and_path_loop + 1], tranparency)
            # Rename the layer
            pdb.gimp_item_set_name(layer_id_list[layer_and_path_loop + 1],str(text[loop]))
            # Get the name of the current layer
            if use_custom: pdb.gimp_item_set_name(layer_id_list[layer_and_path_loop + 1],str(text[loop]))
            temp = pdb.gimp_item_get_name((layer_id_list[layer_and_path_loop + 1]))

            # Add further layer names to the list of text layer names
            layer_name_list.append(temp)

            # Rename the layer to account for multiple occurrences of the same letter
            pdb.gimp_item_set_name(layer_id_list[layer_and_path_loop + 1],str(layer_name_list[layer_and_path_loop + 1]))
            # Set x,y the position of subsequent layers in the image
				
            pdb.gimp_layer_translate(layer_id_list[layer_and_path_loop + 1],f_xpos,text_layer_y_origin )

            if not use_custom:
                # Add further path identifiers to the list of path identifiers
                path_id_list.append(pdb.gimp_vectors_new_from_text_layer(image,layer_id_list[layer_and_path_loop + 1]))
                # Add further names of the list paths names
                # Use loop instead of layer_and_path_loop since we are referencing the original string for the path layer 
                paths_names_list.append(str(joining_text_list[loop]))
                # Insert path
                temp = pdb.gimp_image_insert_vectors(image,path_id_list[layer_and_path_loop + 1],None,-1)
				
                char_vector = pdb.gimp_image_get_vectors_by_name(image, paths_names_list[layer_and_path_loop + 1])
	
            kerning_vector = -1
            if use_custom:
                kerning_name, kerning_name_2 = CurrentChar + NextChar, CurrentChar + NextChar + ".png"
                kerning_vector = pdb.gimp_image_get_vectors_by_name(fontimg,kerning_name) # check for character kerning pair paths without .png
                if kerning_vector == None: kerning_vector = pdb.gimp_image_get_vectors_by_name(fontimg,kerning_name_2) # check for character kerning pair paths with .png	
					
            if char_vector >=0 and kerning_vector <0:
                #kerning_name = CurrentChar + NextChar

                char_vector_name = pdb.gimp_item_get_name(char_vector)
                NumberOfStrokes_in_Path,Char_Vector_StrokeIDs_List = pdb.gimp_vectors_get_strokes(char_vector)
                Char_Vector_Stroke = Char_Vector_StrokeIDs_List[0]
                TypeOfStrokeInChar_Vector,NumberOfPointsInChar_Vector,ListOfControlPoints,IsStrokeClosed = pdb.gimp_vectors_stroke_get_points(char_vector,Char_Vector_Stroke)
                firstX = ListOfControlPoints[2]

                secondX = ListOfControlPoints[len(ListOfControlPoints)-4]
                minX = min(firstX,secondX)
                maxX = max(firstX,secondX)
                vector_char_width = maxX - minX #overwrite char_width to be used as character width.
                add_width = vector_char_width
                offsetX = minX-offset_x #overwrite offsetX with difference of starting X(minX) offset_x.
                if use_custom: custom_width = add_width * custom_scale
                #if use_custom: char_width = add_width * custom_scale # Slab v 0.06 caters for negative kerning with custom fonts
                if use_custom and not vertical: custom_width = add_width * custom_scale ##################### altered for loads of text
                if use_custom and not vertical: f_xpos += (minX * custom_scale) # Slab v 0.06 caters for negative kerning with custom fonts  ##################### altered for loads of text


            if kerning_vector >= 0: 
                NumberOfStrokes_in_Path,Char_Vector_StrokeIDs_List = pdb.gimp_vectors_get_strokes(kerning_vector)
                Char_Vector_Stroke = Char_Vector_StrokeIDs_List[0]
                TypeOfStrokeInChar_Vector,NumberOfPointsInChar_Vector,ListOfControlPoints,IsStrokeClosed = pdb.gimp_vectors_stroke_get_points(kerning_vector,Char_Vector_Stroke)
                firstX = ListOfControlPoints[2]
                secondX = ListOfControlPoints[len(ListOfControlPoints)-4]
                minX = min(firstX,secondX)
                maxX = max(firstX,secondX)
                kerning_width = maxX - minX #overwrite char_width to be used as character width.
                add_width = kerning_width
                offsetX = minX-offset_x #overwrite offsetX with difference of starting X(minX) offset_x.
                if use_custom and not vertical: custom_width = add_width * custom_scale ##################### altered for loads of text
                if use_custom and not vertical: char_width = add_width * custom_scale # Slab v 0.06 caters for negative kerning with custom fonts ##################### altered for loads of text
                if use_custom and not vertical: f_xpos += (minX * custom_scale) # Slab v 0.06 caters for negative kerning with custom fonts ##################### altered for loads of text
					
            if vertical:  # Added for vertical text		
                dist =  f_xpos
                if diagonal: 
                    dist =  (f_xpos ) #calculate center dist of letter before adding to width.
            else:				
                dist =  f_xpos + float(char_width)/2 #calculate center dist of letter before adding to width.

            # Get the x,y position of the required point along the active path to position the characters
            x_point,y_point,SlopeAtThisPoint,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist,0.01)
            #x_point1,y_point1,SlopeAtThisPoint1,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist-5,0.01)
            #x_point2,y_point2,SlopeAtThisPoint2,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist+5,0.01)
            x_point1,y_point1,SlopeAtThisPoint1,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist,0.01)
            x_point2,y_point2,SlopeAtThisPoint2,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist+0.5,0.01)
            if rotate_text:
                if jitter: 
                    user_angle = fn_angle_jitter(original_user_angle,jitter_range)
                    angle = math.radians(user_angle)
                else:
                    angle = math.radians(original_user_angle)
            else:
                if jitter:
                    angle = computeThetaWithSlope([x_point1,y_point1], [x_point2,y_point2],SlopeAtThisPoint)
                    jitter_angle = math.degrees(angle) 
                    jitter_angle = fn_angle_jitter(jitter_angle,jitter_range)
                    angle = math.radians(jitter_angle)
                else:
                    angle = computeThetaWithSlope([x_point1,y_point1], [x_point2,y_point2],SlopeAtThisPoint)

            #angle = computeThetaWithSlope([x_point1,y_point1], [x_point2,y_point2],SlopeAtThisPoint)
            #angle = computeThetaWithSlope([x_point,y_point], [x_point,y_point],SlopeAtThisPoint)
                # Initialise angle variables to be used in trying to compensate for distortions along curved path
            if loop == 0:
                current_angle, prev_angle = math.degrees(angle), math.degrees(angle)
            else:
                current_angle = math.degrees(angle)
								
            curve_compensation,temp_angle_diff = fn_compensate_for_curve_in_path(prev_angle, current_angle, f_size)
            curve_compensation = (curve_compensation * compensate_multiplier)
            curve = curve_compensation
				
            if vertical: # Added for Vertical Text	
                dist =  f_xpos
                if diagonal: 
                    dist =  (f_xpos ) #calculate center dist of letter before adding to width.
            else:				
                dist =  (f_xpos + curve_compensation) + float(char_width)/2 #calculate center dist of letter before adding to width.

            x_point,y_point,SlopeAtThisPoint,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist,0.01)
            x_point1,y_point1,SlopeAtThisPoint1,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist-5,0.01)
            x_point2,y_point2,SlopeAtThisPoint2,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist+5,0.01)
				
            translate_x = x_point - float(char_width)/2
            if char_vector>=0 and use_custom: translate_x += (minX * custom_scale)  # Slab v 0.06 cater for negative kerning with custom fonts
            translate_y = y_point - float(layer_id_list[layer_and_path_loop + 1].height)/2 # ie on the path
			
            #### Alter y position here ####
			
            pdb.gimp_layer_translate(layer_id_list[layer_and_path_loop + 1],translate_x - f_xpos,translate_y - text_layer_y_origin)
												
            x,y = pdb.gimp_drawable_offsets(layer_id_list[layer_and_path_loop + 1])
            #v 2.11 line
            temp = pdb.gimp_item_transform_rotate(layer_id_list[layer_and_path_loop + 1],angle,False,x+float(char_width)/2,y+float(layer_id_list[layer_and_path_loop + 1].height/2)) # rotate original layer on path
						
            if not use_custom:						
                # Delete path
                if (keep_paths == 0):
                    pdb.gimp_image_remove_vectors(image,path_id_list[layer_and_path_loop+1])
            # Merge new character layer to first character layer
            if (keep_layers == 0):
                if loop>0: pdb.gimp_image_merge_down(image, layer_id_list[layer_and_path_loop + 1],0)
            temp = pdb.gimp_displays_flush()

            layer_and_path_loop += 1				

        # Update y position adjustment
        if use_custom:
            text_layer_y_origin = text_layer_y_origin + (font_y_offset - (custom_height / 2))
    	    font_y_offset = (custom_height / 2)
            #f_xpos = f_xpos + custom_width + kern_gap + curve - xpos_adjustment
            f_xpos = f_xpos + custom_width + curve - xpos_adjustment
            if not loop == len(joining_text_list)-1: f_xpos = f_xpos + kern_gap
            if f_xpos < init_f_xpos: f_xpos = init_f_xpos
        else:
            (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(CurrentChar, f_size, PIXELS, str(f_name))
            text_layer_y_origin = text_layer_y_origin + (font_y_offset - fontbox_ascent)
            font_y_offset = fontbox_ascent
			
            f_xpos = f_xpos + fontbox_width + kern_gap + curve - xpos_adjustment
		
        prev_angle = current_angle

        if (CurrentChar == " "):
            
            f_size_adjustment = step_size
            f_size = f_size + f_size_adjustment
            # Stop font size becoming too small/large
            if (f_size<2):
                f_size = 2
                governor += 1
                if governor > 2: loop = len(joining_text_list)
            if (f_size>3000):
                f_size = 3000
                governor += 1
                if governor > 2: loop = len(joining_text_list)
				
            if not use_custom: kern_gap += (f_size * kern_factor) # otherwise it will double the amount of padding for the custom font
            shading_loop += 1
        else:
            if (use_space_to_change_step_size == 0):
                f_size_adjustment = step_size
                f_size = f_size + f_size_adjustment
                if (f_size<2):
                    f_size = 2
                    governor += 1
                    if governor > 2: loop = len(joining_text_list)
                if (f_size>3000):
                    f_size = 3000
                    governor += 1
                    if governor > 2: loop = len(joining_text_list)
                dummy = 0
                #kern_gap = (f_size * kern_factor)
        
        if use_custom: 
            tempbox_width = 0
            if not do_best_size:
                if NextChar <> "":
                    tempbox_layer_name = CurrentChar + ".png"
                    if CurrentChar == " ":
                        tempbox_width = space_width
                        temp_width = space_width
                    else:
                        tempbox_layer = pdb.gimp_image_get_layer_by_name(fontimg,tempbox_layer_name)
                        custom_scale = fn_get_custom_font_scale(f_size, tempbox_layer.height)
							
							
							
                        tempbox_width = tempbox_layer.width * custom_scale
                        fontimg_layer2 = pdb.gimp_image_get_layer_by_name(fontimg, NextChar+".png")
                        temp_width2 = fontimg_layer2.width
						
						
						
                        if vertical: ################# added for loads of text
								
                            find_me = fn_find_layer_in_list(vertical_adjustment_list,tempbox_layer_name)
                            height_adjusted = vertical_adjustment_list[find_me + 4]
                            if NextChar<>"":
                                find_me_next = fn_find_layer_in_list(vertical_adjustment_list,NextChar+".png")
                            tempbox_width = height_adjusted * custom_scale
							
							
							
                            if diagonal:
                                temp_width = (vertical_adjustment_list[find_me + 5])/2.25
                                if NextChar<>"":
                                    NextChar_name = NextChar + ".png"
                                    temp_width2 = (vertical_adjustment_list[find_me_next + 5])/2.25
                                    tempbox_width = (((temp_width)+(temp_width2))/(math.sin(0.78534031413613))) * custom_scale
                                else:
                                    tempbox_width = ((tempbox_width)) * custom_scale
				
					
                    
        else:
            (tempbox_width, tempbox_height, tempbox_ascent, tempbox_descent) = pdb.gimp_text_get_extents_fontname(NextChar, f_size, PIXELS, str(f_name))
								
        NextChr_width = tempbox_width
        loop = loop + 1
		
    if (keep_layers == 0):
        final_layer = pdb.gimp_image_get_active_layer(image)
        label_repeat_info = ""
        if repeat_text: label_repeat_info = " repeat"
        label_colour_info = ""
        if spread_colour: label_colour_info = " shaded"
        if use_gradient_colour: label_colour_info = " gradient used"
        if initial_step_size <> 0: label_fontsize_info += " with " + str(initial_step_size) + "pt steps"
        label_details += label_repeat_info
        label_details += label_colour_info
        label_details += label_fontsize_info
        label_details += " Font: " + label_fontname
        if rotate_text:
            label_rotate = " rotated " + str(user_angle) + str(unichr(176))
            label_details += label_rotate
        if (jitter and jitter_range <> 0):
            label_jitter = " jitter +/- " + str(jitter_range/2) + str(unichr(176))
            label_details += label_jitter
        
        pdb.gimp_item_set_name(final_layer,label_details)
    
        
        if reversed_vector > -1: pdb.gimp_image_remove_vectors(image, reversed_vector)
    return custom_scale
##########################################################################################################

#### End of proc_text_along_path
 	
#####################################################################################################

#######################################################################################################

def fn_get_custom_font_name(filepath):
    search = filepath.rfind("\\")
    part_name = filepath[search + 1:(len(filepath))]
    search = part_name.rfind(".")
    part_name = part_name[0:search]
    return part_name
#######################################################################################################

def fn_get_custom_font_scale(f_size, custom_font_height):
    new_custom_height = f_size * 1.164
    scale = new_custom_height / custom_font_height
    return scale

#####################################################################################################
  
    #### Useful Functions
# -------------------------------------------------------------------------------------
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################

def fn_set_metal_colour(mc):
    # Returns mode for metallic colours and the colour name
    if (mc == 1):
        n_colour = "Gold"
        pdb.gimp_context_set_foreground(lightgold)
        pdb.gimp_context_set_background(darkgold)
        c_mode = 19#SOFTLIGHT
    elif (mc == 2):
        n_colour = "Copper"
        pdb.gimp_context_set_foreground(darkcopper)
        pdb.gimp_context_set_background(lightcopper)
        c_mode = 21#GRAIN MERGE
    elif (mc == 3):
        n_colour = "Brass"
        pdb.gimp_context_set_foreground(darkbrass)
        pdb.gimp_context_set_background(lightbrass)
        c_mode = 19#SOFTLIGHT
    elif (mc == 4):
        n_colour = "Bronze"
        pdb.gimp_context_set_foreground(darkbronze)
        pdb.gimp_context_set_background(lightbronze)
        c_mode = 19#SOFTLIGHT
    else:
        n_colour = "Silver"
        pdb.gimp_context_set_foreground(lightgrey)
        pdb.gimp_context_set_background(midgrey)
        c_mode = 19#SOFTLIGHT

    return c_mode, n_colour

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################
		
def proc_trim_away(img, layer, amount):
    #### Trim off pixels from a layer
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)
	
    #### End of Proc_trim_away
	
######################################################################################################################################

######################################################################################################################################
	
register(
    "python_fu_Custom_Font_WordWall_v001",
    "Provides options for creating a wall of text",
    "Provides options  for creating a wall of text with custom fonts",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2020",
    "<Image>/Filters/Custom Font Tools/Custom Font WordWall v001...",
    "RGB*",
    [
    (PF_DIRNAME, "pf_folder", "Custom font folder to use:", 0),
    #(PF_TOGGLE, "current_font_option", "Use just current font: ", 0),
    #(PF_TOGGLE, "all_font_option", "Use randomly selected fonts: ", 1),
    (PF_TOGGLE, "allow_start_off", "Allow characters before image boundary: ", 0),
    (PF_TOGGLE, "allow_end_off", "Allow characters after image boundary: ", 0),
    (PF_STRING, "pf_string", "Text: ", "Gimp"),
    (PF_SLIDER, "pf_hpadding", "Spacing adjustment \n(between horizontal characters): ", 0, (-50, 100, 1)),
    (PF_SLIDER, "pf_vpadding", "Spacing adjustment \n(between vertical and diagonal characters): ", 0, (-50, 100, 1)),
    (PF_SLIDER, "pf_repeats", "Number of repeats: ", 5, (1, 20, 1)),
    (PF_OPTION, "style", "Text placement style", 0, ("Horizontal", "Vertical", "Diagonal")),
    #(PF_OPTION, "colour", "Text colour style", 0, ("Foreground Colour", "Background colour", "Gradient colour", "Random gradient colour")),
    #(PF_GRADIENT, "pf_current_gradient", "Gradient for text colours: ", 0),
    (PF_TOGGLE, "pf_add_drop", "Add a basic drop shadow (background colour): ", 0),
    (PF_SLIDER, "pf_drop_offset", "Shadow offset: ", 16, (-50, 50, 1)),
    (PF_SLIDER, "pf_drop_blur", "Shadow blur percentage: ", 5, (0, 50, 1)),
    (PF_TOGGLE, "pf_add_shuffle", "Shuffle the layers: ", 0),
    ],
    [],
    do_stuff)

main()
